@@ -71,8 +71,8 @@ struct SnavelyReprojectionError {
     // the camera model that Noah Snavely's Bundler assumes, whereby
     // the camera coordinate system has a negative z axis.
     const T& focal = camera[6];
-    T xp = - focal * p[0] / p[2];
-    T yp = - focal * p[1] / p[2];
+    T xp = - p[0] / p[2];
+    T yp = - p[1] / p[2];
 
     // Apply second and fourth order radial distortion.
     const T& l1 = camera[7];
@@ -81,8 +81,8 @@ struct SnavelyReprojectionError {
     T distortion = T(1.0) + r2  * (l1 + l2  * r2);
 
     // Compute final projected point position.
-    T predicted_x = distortion * xp;
-    T predicted_y = distortion * yp;
+    T predicted_x = focal * distortion * xp;
+    T predicted_y = focal * distortion * yp;
 
     // The error is the difference between the predicted and observed position.
     residuals[0] = predicted_x - T(observed_x);
@@ -100,10 +100,10 @@ struct SnavelyReprojectionError {
 // translation, 1 for focal length and 2 for radial distortion. The
 // principal point is not modeled (i.e. it is assumed be located at
 // the image center).
-struct SnavelyReprojectionErrorWitQuaternions {
+struct SnavelyReprojectionErrorWithQuaternions {
   // (u, v): the position of the observation with respect to the image
   // center point.
-  SnavelyReprojectionErrorWitQuaternions(double observed_x, double observed_y)
+  SnavelyReprojectionErrorWithQuaternions(double observed_x, double observed_y)
       : observed_x(observed_x), observed_y(observed_y) {}
 
   template <typename T>
@@ -128,16 +128,16 @@ struct SnavelyReprojectionErrorWitQuaternions {
     // Compute the center of distortion. The sign change comes from
     // the camera model that Noah Snavely's Bundler assumes, whereby
     // the camera coordinate system has a negative z axis.
-    T xp = - focal * p[0] / p[2];
-    T yp = - focal * p[1] / p[2];
+    T xp = - p[0] / p[2];
+    T yp = - p[1] / p[2];
 
     // Apply second and fourth order radial distortion.
     T r2 = xp*xp + yp*yp;
     T distortion = T(1.0) + r2  * (l1 + l2  * r2);
 
     // Compute final projected point position.
-    T predicted_x = distortion * xp;
-    T predicted_y = distortion * yp;
+    T predicted_x = focal * distortion * xp;
+    T predicted_y = focal * distortion * yp;
 
     // The error is the difference between the predicted and observed position.
     residuals[0] = predicted_x - T(observed_x);
