@@ -32,22 +32,21 @@
 
 #include <cmath>
 #include <vector>
-
-#include <glog/logging.h>
-#include "gmock/gmock.h"
-#include "gtest/gtest.h"
-#include "ceres/mock_log.h"
-#include "ceres/problem_impl.h"
-#include "ceres/program.h"
-#include "ceres/parameter_block.h"
-#include "ceres/residual_block.h"
-#include "ceres/random.h"
 #include "ceres/cost_function.h"
 #include "ceres/internal/scoped_ptr.h"
 #include "ceres/local_parameterization.h"
 #include "ceres/loss_function.h"
+#include "ceres/parameter_block.h"
+#include "ceres/problem_impl.h"
+#include "ceres/program.h"
+#include "ceres/random.h"
+#include "ceres/residual_block.h"
 #include "ceres/sized_cost_function.h"
 #include "ceres/types.h"
+#include "glog/logging.h"
+#include "gmock/gmock.h"
+#include "gmock/mock-log.h"
+#include "gtest/gtest.h"
 
 using testing::AllOf;
 using testing::AnyNumber;
@@ -340,7 +339,11 @@ void ParameterBlocksAreEquivalent(const ParameterBlock*  left,
 }
 
 TEST(GradientCheckingProblemImpl, ProblemDimensionsMatch) {
-  double x[3], y[4], z[5], w[4];
+  // Parameter blocks with arbitrarily chosen initial values.
+  double x[] = {1.0, 2.0, 3.0};
+  double y[] = {4.0, 5.0, 6.0, 7.0};
+  double z[] = {8.0, 9.0, 10.0, 11.0, 12.0};
+  double w[] = {13.0, 14.0, 15.0, 16.0};
 
   ProblemImpl problem_impl;
   problem_impl.AddParameterBlock(x, 3);
